/*
 * Copyright (c) 2021 NetEase, Inc.  All rights reserved.
 * Use of this source code is governed by a MIT license that can be
 * found in the LICENSE file.
 */

import Foundation
import NIMSDK

enum EventSubscribeType: String {
    case SubscribeEvent = "registerEventSubscribe"
    case UnSubscribeEvent = "unregisterEventSubscribe"
    case BatchUnSubscribeEvent = "batchUnSubscribeEvent" // iOS无此接口
    case PublishEvent = "publishEvent"
    case QuerySubscribeEvent = "querySubscribeEvent"
}

class FLTEventSubscribeService: FLTBaseService, FLTService {
    
    override init() {
        super.init()
        NIMSDK.shared().subscribeManager.add(self)
    }
    
    deinit {
        NIMSDK.shared().subscribeManager.remove(self)
    }
    
    // MARK: Service Protocol
    func serviceName() -> String {
        return ServiceType.EventSubscribeService.rawValue
    }
    
    func onMethodCalled(_ method: String, _ arguments: [String : Any], _ resultCallback: ResultCallback) {
        switch method {
        case EventSubscribeType.SubscribeEvent.rawValue:
            subscribeEvent(arguments, resultCallback)
        case EventSubscribeType.UnSubscribeEvent.rawValue:
            unSubscribeEvent(arguments, resultCallback)
        case EventSubscribeType.PublishEvent.rawValue:
            publishEvent(arguments, resultCallback)
        case EventSubscribeType.QuerySubscribeEvent.rawValue:
            querySubscribeEvent(arguments, resultCallback)
        default:
            resultCallback.notImplemented()
            break
        }
    }
    
    func register(_ nimCore: NimCore) {
        self.nimCore = nimCore
        nimCore.addService(self)
    }
    
    // MARK: Public Method
    func subscribeEvent(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let request = NIMSubscribeRequest.yx_model(with: arguments) else {
            let result = NimResult(nil, -1, "subscribeEvent but the args are invalid")
            resultCallback.result(result.toDic())
            return
        }
        if request.type < 0 {
            let result = NimResult(nil, -1, "subscribeEvent eventType must be greater than 0")
            resultCallback.result(result.toDic())
            return
        }
        NIMSDK.shared().subscribeManager.subscribeEvent(request) { error, ret in
            guard let nserror = error as NSError? else {
                if (ret != nil && ret!.count > 0) {
                    let result = NimResult(ret, 0, nil)
                    resultCallback.result(result.toDic())
                } else {
                    let result = NimResult(nil, 0, nil)
                    resultCallback.result(result.toDic())
                }
                return
            }
            let result = NimResult(nil, NSNumber(value: nserror.code), nserror.description)
            resultCallback.result(result.toDic())
        }
    }
    
    func unSubscribeEvent(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let request = NIMSubscribeRequest.yx_model(with: arguments) else {
            let result = NimResult(nil, -1, "unSubscribeEvent but the args are invalid")
            resultCallback.result(result.toDic())
            return
        }
        if request.type < 0 {
            let result = NimResult(nil, -1, "unSubscribeEvent eventType must be greater than 0")
            resultCallback.result(result.toDic())
            return
        }
        NIMSDK.shared().subscribeManager.unSubscribeEvent(request) { error, ret in
            guard let nserror = error as NSError? else {
                if (ret != nil && ret!.count > 0) {
                    let result = NimResult(ret, 0, nil)
                    resultCallback.result(result.toDic())
                } else {
                    let result = NimResult(nil, 0, nil)
                    resultCallback.result(result.toDic())
                }
                return
            }
            let result = NimResult(nil, NSNumber(value: nserror.code), nserror.description)
            resultCallback.result(result.toDic())
        }
    }
    
    func publishEvent(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let event = NIMSubscribeEvent.createSubscribeEvent(arguments) else {
            let result = NimResult(nil, -1, "publishEvent but the args are invalid")
            resultCallback.result(result.toDic())
            return
        }
        if event.type < 0 {
            let result = NimResult(nil, -1, "publishEvent eventType must be greater than 0")
            resultCallback.result(result.toDic())
            return
        }
        NIMSDK.shared().subscribeManager.publishEvent(event, completion: { error, ret in
            guard let nserror = error as NSError? else {
                let result = NimResult(ret, 0, nil)
                resultCallback.result(result.toDic())
                return
            }
            let result = NimResult(nil, NSNumber(value: nserror.code), nserror.description)
            resultCallback.result(result.toDic())
        })
    }
    
    func querySubscribeEvent(_ arguments: [String : Any], _ resultCallback: ResultCallback) {
        guard let request = NIMSubscribeRequest.yx_model(with: arguments) else {
            let result = NimResult(nil, -1, "querySubscribeEvent but the args are invalid")
            resultCallback.result(result.toDic())
            return
        }
        if request.type < 0 {
            let result = NimResult(nil, -1, "querySubscribeEvent eventType must be greater than 0")
            resultCallback.result(result.toDic())
            return
        }
        NIMSDK.shared().subscribeManager.querySubscribeEvent(request) { error, ret in
            guard let nserror = error as NSError? else {
                if (ret == nil) {
                    let result = NimResult(nil, 0, nil)
                    resultCallback.result(result.toDic())
                } else {
                    var array: [[String : Any?]] = [[String : Any?]]()
                    for subscribeResult in ret! {
                        if let s = subscribeResult as? NIMSubscribeResult,
                           let dic = s.yx_modelToJSONObject() as? [String : Any?] {
                            array.append(dic)
                        }
                    }
                    let result = NimResult(array.count > 0 ? array : nil, 0, nil)
                    resultCallback.result(result.toDic())
                }
                return
            }
            let result = NimResult(nil, NSNumber(value: nserror.code), nserror.description)
            resultCallback.result(result.toDic())
        }
    }
}

extension FLTEventSubscribeService : NIMEventSubscribeManagerDelegate {
    func onRecvSubscribeEvents(_ events: [Any]) {
        var array: [[String : Any?]] = [[String : Any?]]()
        for evnet in events {
            guard let e = evnet as? NIMSubscribeEvent else {
                break
            }
            array.append(e.yx_modelToJSONObject() as! [String : Any?])
        }
        notifyEvent(serviceName(), "observeEventChanged", ["eventList":array])
    }
}
